'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-08821612.js');

class Parser {
  static Parse(xmlstr) {
    if (typeof DOMParser !== 'undefined') {
      let domDoc = (new DOMParser()).parseFromString(xmlstr, 'text/xml');
      // let's check for errors
      if (domDoc.querySelector('parsererror')) {
        throw new Error('XML parsing error');
      }
      return domDoc;
    }
    return null;
  }
}

class CommentRenderer {
  render(node) {
    const comment = node;
    if (!comment) {
      return null;
    }
    return (index.h("div", { class: "comment" },
      "<--\u00A0",
      comment.nodeValue.trim(),
      "\u00A0-->"));
  }
}

class CDataRenderer {
  render(node) {
    if (!node) {
      return null;
    }
    const data = unescape(`<![CDATA[${node.textContent.trim()}]]>`);
    return (index.h("ul", null,
      index.h("li", null,
        index.h("div", { class: "cdata" }, data))));
  }
}

class Renderer {
  constructor(renderers = new Array()) {
    this.renderers = renderers;
  }
  renderNode(node) {
    const renderer = this.getRenderer(node);
    if (renderer) {
      return renderer.render(node);
    }
    return null;
  }
  getRenderer(node) {
    if (!node) {
      return null;
    }
    if (node.nodeName.toLowerCase() === "#comment") {
      let cr = this.renderers.find(e => e instanceof CommentRenderer);
      if (!cr) {
        cr = new CommentRenderer();
        this.renderers.push(cr);
      }
      return cr;
    }
    else if (node.nodeName.toLowerCase() === `#cdata-section`) {
      let r = this.renderers.find(e => e instanceof CDataRenderer);
      if (!r) {
        r = new CDataRenderer();
        this.renderers.push(r);
      }
      return r;
    }
    return null;
  }
}

const xmlViewerComponentCss = "code{margin:.1rem .25rem 1rem .25rem;padding:0;color:black;font-family:monospace, monospace;text-shadow:0 1px #fff;direction:ltr;text-align:left;word-spacing:normal;word-break:normal;word-wrap:normal;line-height:1.2;-moz-tab-size:2;-o-tab-size:2;tab-size:2;-webkit-hyphens:none;-moz-hyphens:none;-ms-hyphens:none;hyphens:none}code ul{list-style-type:none;margin:0;padding-left:1rem}code div.element{font-weight:bold;color:darkred;display:inline-block}code div.element-content{font-weight:normal;display:inline-block}code span.attr{font-weight:bold;color:#535353}code span.attr-value{font-weight:normal}code div.comment{color:darkgreen;font-style:italic}code div.cdata{color:lightgray}";

const XmlViewerComponent = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.useUnescapedNodeValue = true;
    this.nodeRenderer = new Renderer();
  }
  componentDidLoad() {
  }
  xmlPropChanged() {
    this.render();
  }
  useUnescapedNodeValuePropChanged() {
    this.render();
  }
  // for FireFox and maybe Edge.
  toArray(obj) {
    let arr = [];
    for (let i = 0; i < obj.length; i++) {
      arr.push(obj[i]);
    }
    return arr;
  }
  prepareXml() {
    let xdoc = Parser.Parse(this.xml.trim());
    return xdoc;
  }
  renderAttribute(attribute) {
    return (index.h("span", null, "\u00A0", index.h("span", { class: "attr" }, attribute.name, "=\"", index.h("span", { class: "attr-value" }, attribute.value), "\"")));
  }
  renderNodeValue(nodeValue) {
    if (!nodeValue) {
      return null;
    }
    const val = this.useUnescapedNodeValue ? unescape(nodeValue.trim()) : nodeValue.trim();
    if (val.length === 0) {
      return null;
    }
    if (val.length > 150) {
      return (index.h("ul", null, index.h("li", null, val)));
    }
    return (index.h("span", null, val));
  }
  renderNode(node) {
    return this.nodeRenderer.renderNode(node);
  }
  // rendering node. This function calls to itself in recursion way in case of child nodes
  renderElement(element) {
    if (!element) {
      return null;
    }
    // element.attributes, childNodes and children has no iterator so I have to turn it into an array
    const attributes = this.toArray(element.attributes);
    const childNodes = this.toArray(element.childNodes);
    const children = this.toArray(element.children);
    let nodeValue = element.firstChild ? element.firstChild.nodeValue : null;
    nodeValue = nodeValue ? nodeValue.trim() : null;
    return (index.h("ul", null, index.h("li", null, index.h("div", { class: "element" }, "<", element.nodeName, attributes.map(a => this.renderAttribute(a)), ">"), index.h("div", { class: "element-content" }, this.renderNodeValue(nodeValue)), children.map(c => this.renderElement(c)), childNodes.map(c => this.renderNode(c)), index.h("div", { class: "element" }, "</", element.nodeName, ">"))));
  }
  render() {
    if (this.xml === null || this.xml === undefined) {
      return null;
    }
    if (this.xml.length === 0) {
      return null;
    }
    let xdoc = null;
    try {
      xdoc = this.prepareXml();
    }
    catch (e) {
      console.error(e);
    }
    if (!xdoc || !xdoc.documentElement.childNodes) {
      return (index.h("code", null, this.xml));
    }
    return (index.h("p", null, index.h("code", null, this.renderElement(xdoc.documentElement))));
  }
  static get watchers() { return {
    "xml": ["xmlPropChanged"],
    "useUnescapedNodeValue": ["useUnescapedNodeValuePropChanged"]
  }; }
};
XmlViewerComponent.style = xmlViewerComponentCss;

exports.xml_viewer_component = XmlViewerComponent;
